C **************************************************************** 
C                                                                  
C MAIN - Main program - driver routine                             
C   Parameters :                                                   
C                                                                  
C **************************************************************** 

C -- STEM INTEGRATION WITH RADAU5.F --
        PROGRAM STEM
      INCLUDE 'agg.h'
C --- PARAMETERS FOR RADAU5 (FULL JACOBIAN)
        PARAMETER (ND=NVAR,LWORK=5*ND*ND+12*ND+20,LIWORK=3*ND+20)
        PARAMETER (LRCONT=4*ND+4)
        COMMON /CONT/ICONT(4),RCONT(LRCONT)
C --- DECLARATIONS
        DIMENSION  WORK(LWORK),IWORK(LIWORK)
        COMMON/STAT/NFCN,NJAC,NSTEP,NACCPT,NREJCT,NDEC,NSOL
        EXTERNAL FUN,JAC,SOLOUT
        DIMENSION VMIN(ND)
        DATA VMIN/5.9e+10,1.8e+09,2.8e+08,
     *  5.0e+09,6.1e+00,1.0e-04,
     *  1.0e-04,2.5e+09,4.3e+01,1.0e-04,1.5e+05,2.5e+10,
     *  1.0e-04,6.6e+07,1.0e-04,1.1e+04,3.2e+04,5.2e+07,
     *  1.7e+02,5.4e+04,2.8e+06,4.6e+05,4.5e+09,1.5e-01,
     *  1.0e-04,9.1e+07,5.0e+07,1.0e-04,6.3e+05,1.1e-02,
     *  1.0e-04,1.8e+08,4.9e+12,6.6e+03,4.1e+04,5.0e+06,
     *  2.5e+10,8.2e+05,1.0e-04,2.1e+10,1.0e-04,9.2e+08,
     *  9.6e+05,1.5e+06,7.7e+08,3.2e+07,1.0e-04,1.4e+10,
     *  2.5e+09,1.0e-04,2.1e+08,3.1e+02,5.6e+04,1.1e+12,
     *  4.3e+07,2.5e+09,1.0e+03,5.1e+08,1.0e-04,2.1e+03,
     *  8.8e+04,5.1e+05,5.7e+03,2.7e+01,4.9e+09/


C --- DIMENSION OF THE SYSTEM
        N=NVAR
C --- OUTPUT FILE ----
      OPEN(UNIT=10,FILE='Radau.m')

        write(6,*) 'RADAU5. Rtol='
        read(5,*) rtols
        DT = 3600.

        IJAC=1
        MLJAC=N     
        IMAS=0     
        IOUT=0     

        do i=1,n
          ATOL(i)=DMAX1(1.0D-1, 1.0D-2*rtols*VMIN(i) )
          RTOL(i)=rtols
        end do  
        ITOL=1
        
        
C ==== initial values ====
      CM = 1./CFACTOR
      TEMP = 288.15
      CALL INITVAL()
      
C --- ENDPOINT OF INTEGRATION
       NDAYS = 5
       TSTART = 12.D0*3600.D0
       TEND = TSTART + 24.D0*3600.D0*NDAYS
C --- INITIAL STEP SIZE
        H=0.1 
C --- SET DEFAULT VALUES 
        DO 10 I=1,20
        IWORK(I)=0
  10    WORK(I)=0.D0


       TIME=TSTART
C --- PRINT  INITIAL CONDITIONS
        write(6,95) 0.,Var(I_O3)*CM,
     *            -log10(Var(I_HPLUSaq)*5.53526E-13)

        write(10,99) 0., (VAR(i), i=1,nvar)

        DIV = 3600./DT

        DO WHILE (TIME.LT.TEND)

C --- Add emissions
        VAR(I_NO)   = VAR(I_NO)   + 0.01*CFACTOR /DIV
        VAR(I_NO2)  = VAR(I_NO2)  + 0.01*CFACTOR /DIV
        VAR(I_ISOP) = VAR(I_ISOP) + 0.05* CFACTOR /DIV

C --- CALL OF THE SUBROUTINE RADAU5 
        CALL RADAU5(NVAR,FUN,TIME,VAR,TIME+DT,H,
     &                  RTOL,ATOL,ITOL,
     &                  JAC,IJAC,MLJAC,MUJAC,
     &                  FUN,IMAS,MLMAS,MUMAS,
     &                  SOLOUT,IOUT,
     &                  WORK,LWORK,IWORK,LIWORK,RPAR,IPAR,IDID)
     

C --- PRINT  SOLUTION
        if (time.le.tend) then
        do i=1,NVAR
          Var(i) = dmax1(0.D0, Var(i))
        end do
        
        write(10,99) (TIME-Tstart)/3600.D0,  (VAR(i), i=1,nvar)
        end if
        percent=100.D0*(time-tstart)/(tend-tstart)
C       MCM -> MOL/L by * with    5.53526E-13
        write(6,95) percent,Var(I_O3)*CM,
     *            -log10(Var(I_HPLUSaq)*5.53526E-13)
 95     format(F7.2,' % DONE. O3 = ',F10.4,'  pH = ',F7.2)       
 99     FORMAT(100(X,E24.16))
         END DO
C        write(10,*) '];'



C --- PRINT STATISTICS
        WRITE (6,91) NFCN,NJAC,NSTEP,NACCPT,NREJCT,NDEC,NSOL
 91     FORMAT(' fcn=',I5,' jac=',I4,' step=',I4,
     &        ' accpt=',I4,' rejct=',I3,' dec=',I4,
     &        ' sol=',I5)
        STOP
        END
C
C

        SUBROUTINE FUN (N, T, V, FCT, RPAR, IPAR)
      INCLUDE 'agg.h'
        DIMENSION V(NVAR), FCT(NVAR)
        TOLD = TIME
        TIME = T
        CALL update_sun()
        CALL update_rconst()
        TIME = TOLD
        CALL F_VAR(V, Rad, FCT) 
        RETURN
        END

        SUBROUTINE JAC (N, T, V, JV, NROWPD, RPAR, IPAR)
      INCLUDE 'agg.h'
        REAL*8 V(NVAR), JV(NVAR,NVAR)
        TOLD = TIME
        TIME = T
        CALL update_sun()
        CALL update_rconst()
        TIME = TOLD
        DO i=1,nvar
           DO j=1,nvar
              JV(i,j) = 0.
           END DO
        END DO      
        CALL JACVAR(V, Rad, JV) 
        RETURN
        END
        
C End of MAIN function                                             
C **************************************************************** 

