
      SUBROUTINE Ros3(N,T,Tnext,Hmin,Hmax,
     +                   y,AbsTol,RelTol,
     +                   Info,FUN,JAC_SP)

      REAL*8     K1(NVAR), K2(NVAR), K3(NVAR)
      REAL*8     F1(NVAR), JAC(LU_NONZERO_V)
      REAL*8     Hmin,Hmax,ghinv,uround
      REAL*8     y(NVAR), ynew(NVAR)
      REAL*8     AbsTol(NVAR), RelTol(NVAR)
      REAL*8     T, Tnext, H, Hold, Tplus
      REAL*8     ERR, factor, facmax
      INTEGER    n,nfcn,njac,Naccept,Nreject,i,j
      INTEGER    Info(5)
      LOGICAL    IsReject,Autonom
      EXTERNAL   FUN, JAC_SP

c     Initialization of counters, etc.
      Autonom = Info(1) .EQ. 1
      uround = 1.d-15
      dround = dsqrt(uround)
      H = DMAX1(1.d-8, Hmin)
      Tplus = T
      IsReject = .false.
      Naccept  = 0
      Nreject  = 0
      Nfcn     = 0
      Njac     = 0
      gam=   1.d0 - sqrt(2.d0)/2.d0


C === Starting the time loop ===      
 10    continue  
       Tplus = T + H
       if ( Tplus .gt. Tnext ) then
          H = Tnext - T
          Tplus = Tnext
       end if

       call JAC_SP(NVAR, T, y, JAC)
       Njac = Njac+1
       gH = gam*H
       do 15 j=1,LU_NONZERO_V
         JAC(j) = - gam*H*JAC(j) 
 15    continue
       do 20 j=1,NVAR
         JAC(lu_diag_v(j)) = JAC(lu_diag_v(j)) - gHinv
 20    continue
       call DECOMP (NVAR, JAC, ier)

       if (ier.ne.0) then
         if ( H.gt.Hmin) then
            H = 5.0d-1*H
            go to 10
         else
            print *,'IER <> 0, H=',H
            stop
         end if      
       end if  
       
       call FUN(NVAR, T, y, F1)

C ====== NONAUTONOMOUS CASE ===============
       IF (.not. Autonom) THEN
         tau = dsign(dround*dmax1( 1.0d-6, dabs(T) ), T)
         call FUN(NVAR, T+tau, y, K2)
         nfcn=nfcn+1
         do 30 j = 1,NVAR
           K3(j) = ( K2(j)-F1(j) )/tau
 30      continue
 
C ----- STAGE 1 (NONAUTONOMOUS) -----
         x1 = g1*H
         do 35 j = 1,NVAR
           K1(j) =  F1(j) + x1*K3(j)
 35      continue
         call SOLVE (JAC, K1)
      
C ----- STAGE 2 (NONAUTONOMOUS) -----
       do 40 j = 1,NVAR
         ynew(j) = y(j) + K1(j) 
 40    continue
       call FUN(NVAR, T+gam*H, ynew, F1)
       nfcn=nfcn+1
       x1 = c21/H
       x2 = g2*H
       do 45 j = 1,NVAR
         K2(j) = F1(j) + x1*K1(j) + x2*K3(j)
 45    continue
       call SOLVE (JAC, K2)
       
C ----- STAGE 3  (NONAUTONOMOUS) -----
       x1 = c31/H
       x2 = c32/H
       x3 = g3*H
       do 50 j = 1,NVAR
         K3(j) = F1(j) + x1*K1(j) + x2*K2(j) + x3*K3(j)
 50    continue
       call SOLVE (JAC, K3)


C ====== AUTONOMOUS CASE ===============
       ELSE

C ----- STAGE 1 (AUTONOMOUS) -----
         do 60 j = 1,NVAR
           K1(j) =  F1(j) 
 60      continue
         call SOLVE (JAC, K1)
      
C ----- STAGE 2 (AUTONOMOUS) -----
       do 65 j = 1,NVAR
         ynew(j) = y(j) + K1(j) 
 65    continue
       call FUN(NVAR, T + gam*H, ynew, F1)
       nfcn=nfcn+1
         x1 = c21/H
         do 70 j = 1,NVAR
           K2(j) = F1(j) + x1*K1(j) 
 70      continue
         call SOLVE (JAC, K2)
       
C ----- STAGE 3  (AUTONOMOUS) -----
       x1 = c31/H
       x2 = c32/H
       do 90 j = 1,NVAR
         K3(j) = F1(j) + x1*K1(j) + x2*K2(j)
 90    continue
       call SOLVE (JAC, K3)

       END  IF ! Autonomous

C ---- The Solution ---

       do 120 j = 1,NVAR
         ynew(j) = y(j) + b1*K1(j) + b2*K2(j) + b3*K3(j) 
 120   continue


C ====== Error estimation ========

        ERR=0.d0
        do 130 i=1,NVAR
           ytol = AbsTol(i) + RelTol(i)*dabs(ynew(i))
           ERR=ERR+((d1*K1(i)+d2*K2(i)+d3*K3(i))/ytol)**2
 130    continue      
        ERR = DMAX1( uround, DSQRT( ERR/NVAR ) )

C ======= Choose the stepsize ===============================

        factor = 0.9/ERR**(1.d0/3.d0)
        if (IsReject) then
            facmax=1.0
        else
            facmax=10.0
        end if 
        factor = DMAX1( 1.0D-1, DMIN1(factor,facmax) )
        Hold = H
        H = DMIN1( Hmax, DMAX1(Hmin,factor*H) )    

C ======= Rejected/Accepted Step ============================

        if ( (ERR.gt.1).and.(Hold.gt.Hmin) ) then
          IsReject = .true.
          Nreject  = Nreject+1
        else
          IsReject = .false.
          do 140 i=1,NVAR
             y(i)  = ynew(i)
 140      continue 
          T = Tplus     
          Naccept = Naccept+1
        end if

C ======= End of the time loop ===============================
      if ( T .lt. Tnext ) go to 10
 
     
      
C ======= Output Information =================================
      Info(2) = Nfcn
      Info(3) = Njac
      Info(4) = Naccept
      Info(5) = Nreject
      
      RETURN 
      END        

  
  
        SUBROUTINE FUN(N, T, Y, P)
        INCLUDE 'sparse.h'
        REAL*8   T, Told
        REAL*8   Y(NVAR), P(NVAR)
        Told = TIME
        TIME = T
        CALL UPDATE_SUN()
        CALL UPDATE_PHOTO()
        TIME = Told
        CALL F_VAR( Y, RAD, P )
        RETURN
        END

        SUBROUTINE JAC_SP(N, T, Y, J)
        INCLUDE 'sparse.h'
        REAL*8   Told, T
        REAL*8   Y(NVAR), J(LU_NONZERO_V)
        Told = TIME
        TIME = T
        CALL UPDATE_SUN()
        CALL UPDATE_PHOTO()
        TIME = Told
        CALL JACVAR_SP( Y, RAD, J )
        RETURN
        END
